#ifndef MAPEDITOR_H
#define MAPEDITOR_H

#include <QGraphicsView>

#include "attribute.h"
#include "ornamentedobj.h"
#include "selection.h"		// FIXME should not be needed
#include "settings.h"
#include "vymmodel.h"


/*! \brief Main widget in vym to display and edit a map */

class MapEditor : public QGraphicsView  {	
    Q_OBJECT

public:
    MapEditor(VymModel *vm);
	~MapEditor();
	VymModel* getModel();
	QGraphicsScene * getScene();

public:
    void print();				//!< Print the map
	void setAntiAlias (bool);	//!< Set or unset antialiasing
	void setSmoothPixmap(bool); //!< Set or unset smoothing of pixmaps
public:
	void toggleStandardFlag(QString);

	void updateSelection();				// update geometry of selection

	AttributeTable* attributeTable();
    void testFunction1();				// just testing new stuff
    void testFunction2();				// just testing new stuff

public slots:
	void editHeading();
	void editHeadingFinished();
private:
	bool editingHeading;
	QLineEdit *lineEdit;

protected:
	virtual void contextMenuEvent ( QContextMenuEvent *e );
    virtual void keyPressEvent(QKeyEvent*);
    virtual void keyReleaseEvent(QKeyEvent*);
    virtual void mousePressEvent(QMouseEvent*);
    virtual void mouseMoveEvent(QMouseEvent*);
    virtual void mouseReleaseEvent(QMouseEvent*);
    virtual void mouseDoubleClickEvent(QMouseEvent*);
    virtual void resizeEvent( QResizeEvent * );

	void dragEnterEvent (QDragEnterEvent *);
	void dragMoveEvent (QDragMoveEvent *);
	void dragLeaveEvent (QDragLeaveEvent *);
	void dropEvent (QDropEvent *);


private:
	QGraphicsScene *mapScene;
	VymModel *model;			//!< Vym Map, includding several mapCenters

	bool adjustCanvasRequested;	// collect requests until end of user event
	BranchObj *editingBO;		// entering Text into BO

    QCursor HandOpenCursor;		// cursor while moving canvas view
	QCursor PickColorCursor;	// cursor while picking color 
	QCursor CopyCursor;			// cursor while picking color 
	QCursor XLinkCursor;		// cursor while picking color 
	bool pickingColor;
	bool drawingLink;			// true while creating a link
	bool copyingObj;			// true while creating a link
	XLinkObj* tmpXLink;

	MapObj* movingObj;				// moving a MapObj
	MapObj* linkingObj_src;			// part of a link
    QPointF movingObj_orgPos;		// org. pos of mouse before move
    QPointF movingObj_orgRelPos;	// org. relative pos of mouse before move
    QPointF movingObj_start;		// rel. pos of mouse to absPos 
    QPointF movingCont_start;		// inital pos of moving Content or
    QPointF movingVec;				// how far has Content moved

	QPointF contextMenuPos;			// position where context event was triggered

    QPrinter* printer;				// Printing

	AttributeTable *attrTable;

	bool printFrame;			// Print frame around map
	bool printFooter;			// Print footer below map

	QPoint exportOffset;		// set before export, used in save

	Selection xelection;					// FIXME only here to get rid of some gcc errors...
	QString getName(const LinkableMapObj*);	// FIXME see above, renamed to getObjectName in VymModel...
};
#endif

